#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <dos.h>
#include <stdlib.h>
#include <time.h>

#include "busy.h"

#ifdef __OS2__
#define TIMEZONE _timezone
#else
#define TIMEZONE timezone
#endif

void brakeParser()
{
  if( getenv( "TZ" )==NULL )
  {
    printf( "*Fatal* TZ environiment variable wanted.\n"
	    "        Sorry! Unable to continue. :-(\n" );
    returnToDos( 1 );
  }

  tzset();

  struct SessionInfo
  {
    unsigned short zone, net, node, point;
    char type;
    char protocol;
    char crc;
    unsigned long elapsed;
    unsigned long netSend, netRecv,
		  xmSend,  xmRecv,
		  filesSend, filesRecv;
  };

  SessionInfo sesInfo;

  struct
  {
    unsigned long stamp;
    unsigned char code;
  } header;

  long elapsed;

  while( !feof(logFile) && !ferror(logFile) )
  {
    memset( &header, 0, sizeof( header ) );

    int len = fread( &header, 1, sizeof( header ), logFile );
    progressBar();

    if( len!=sizeof( header ) )
      goto need_abort;

    if( header.code==5 || header.code==6 )
    {
      if( fread( &sesInfo, 1, sizeof(sesInfo), logFile )!=sizeof(sesInfo) )
      {
	printf( "*Error* Incomplete input file at record %lu\n", logLines );

  need_abort:
	header.code = 255;
	return;
      }
    }
    else
      if( header.code<5 )
	if( fread( &elapsed, 1, sizeof(elapsed), logFile )!=sizeof(elapsed) )
	  goto need_abort;

    logLines++;

//    header.stamp -= TIMEZONE;

    if( header.stamp<=hystHeader.date )
    {
      skipLines++;
      startTime = header.stamp;
      continue;
    }

//    struct tm *tmm = gmtime( (long *)&header.stamp );
    struct tm *tmm = localtime( (long *)&header.stamp );
    strftime( logEnd, sizeof(logEnd), "%d %b %H:%M:%S", tmm );
    logEnd[ sizeof(logEnd)-1 ] = 0;

    if( !*logStart )
      strcpy( logStart, logEnd );

    endTime = header.stamp;

    if( !startTime )
      startTime = endTime;

    stamp = endTime;
    actionStarted = stamp;

    static char buf[32],
		*protocolName[4] = { "Zmodem", "ZedZap", "DirZap", "Hydra" };
    index curNode;

    switch( header.code )
    {
    case 255:
      if( ferror( logFile ) )
	printf( "*Error* Can't read file at record %lu\n", logLines );
      break;

    case 7: 			// Empty stamp
      break;

    case 0:			// Answer
      totalAnswering += registerTime( elapsed, Answering );
      inSessions++;
      break;

    case 1:			// Dial
      totalDialing += registerTime( elapsed, Dialing );
      outSessions++;
      break;

    case 2:			// terminal
      totalTerminal += registerTime( elapsed, terminalCalled );
      break;

    case 3:			// bbs
      totalBBS += registerTime( elapsed, BBS );
      break;

    case 4:			// external
      totalExternal += registerTime( elapsed, Executing );
      break;

    case 5:			// income session
    case 6:			// outcome session
      for( curNode=0; curNode<totalNodes; curNode++ )
	if( nodesInfo[curNode]->point==sesInfo.point &&
	    nodesInfo[curNode]->node==sesInfo.node &&
	    nodesInfo[curNode]->net==sesInfo.net &&
	    nodesInfo[curNode]->zone==sesInfo.zone )
	  break;

      if( curNode==totalNodes )
      {

	if( totalNodes==MaxItems )
	{
	  printf( "*Fatal* Too many nodes! Support up to %u nodes.\n"
		  "        Sorry! Program aborted at record %lu :(\n",
		  MaxItems, logLines );
	  returnToDos( 1 );
	}

	curNode=totalNodes++;

	nodesInfo = (NodeInfo **)realloc( nodesInfo, totalNodes*sizeof(NodeInfo **) );
	checkPtr( nodesInfo );

	NodeInfo *newNode( new NodeInfo );
	checkPtr( newNode );

	memset( newNode, 0, sizeof(NodeInfo) );
	nodesInfo[curNode] = newNode;

	static char domain[] = "FidoNet";

	newNode->domain = new char[ strlen(domain) +1 ];
	checkPtr( newNode->domain );
	strcpy( newNode->domain, domain );

	newNode->zone=sesInfo.zone;
	newNode->net=sesInfo.net;
	newNode->node=sesInfo.node;
	newNode->point=sesInfo.point;
      }

      if( header.code==5 )
      {
	totalIncome += registerTime( sesInfo.elapsed, Incoming );
	nodesInfo[curNode]->income += sesInfo.elapsed;
	inHand++;
	nodesInfo[curNode]->inSession++;
      }
      else
      {
	totalOutcome += registerTime( sesInfo.elapsed, Outcoming );
	nodesInfo[curNode]->outcome += sesInfo.elapsed;
	outHand++;
	nodesInfo[curNode]->outSession++;
      }

      totalBytesIn += sesInfo.netRecv+sesInfo.xmRecv+sesInfo.filesRecv;
      inMail += sesInfo.netRecv;
      inXmail += sesInfo.xmRecv;
      inFiles += sesInfo.filesRecv;

      totalBytesOut += sesInfo.netSend+sesInfo.xmSend+sesInfo.filesSend;
      outMail += sesInfo.netSend;
      outXmail += sesInfo.xmSend;
      outFiles += sesInfo.filesSend;

      nodesInfo[curNode]->received += sesInfo.netRecv+sesInfo.xmRecv+sesInfo.filesRecv;
      nodesInfo[curNode]->inMail += sesInfo.netRecv;
      nodesInfo[curNode]->inXmail += sesInfo.xmRecv;
      nodesInfo[curNode]->inFiles += sesInfo.filesRecv;

      nodesInfo[curNode]->sent += sesInfo.netSend+sesInfo.xmSend+sesInfo.filesSend;
      nodesInfo[curNode]->outMail += sesInfo.netSend;
      nodesInfo[curNode]->outXmail += sesInfo.xmSend;
      nodesInfo[curNode]->outFiles += sesInfo.filesSend;

      nodesInfo[curNode]->ioTime += sesInfo.elapsed;

      if( sesInfo.protocol==4 )
        strcat( buf, "Hydra" );
      else
        sprintf( buf, "%s-%d", sesInfo.protocol<3 ? protocolName[sesInfo.protocol]
  						  : "*Unknown*",
			       int( sesInfo.crc ) );
      registerIO( buf );

      switch( sesInfo.type )
      {
      case 2:
	sesEMSI++;
	break;

      case 1:
	sesWaZOO++;
	break;

      case 0:
	sesFTS0001++;
	break;

      default:
	printf( "*Warning* Unknown session type at record %lu, forced to WaZOO\n",
		logLines );
	sesWaZOO++;
      }
      break;

    default:
      printf( "*Error* Illegal record type %d at record %lu\n", int(header.code), logLines );
    }
  }

  progressBar();
}

