#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <string.h>
#include <alloc.h>

#include "busy.h"

extern int optopt, optind;
extern char *optarg;
extern "C" int getopt( int, char * const *, const char * );

extern const char *finitaLaComedia;

void runStat();
void makeHeaderCRC();
char *newStr( char *s );

static near void registerIOCount( char *protocol, big count )
{

  for( int curIO=0; curIO<hystTotalIO; curIO++ )
    if( !stricmp( hystIoInfo[curIO]->name, protocol ) )
      break;

  if( curIO==hystTotalIO )
  {

    if( hystTotalIO==MaxItems )
    {
      printf( "*Fatal* Too many protocols! Support up to %u protocols.\n",
              MaxItems );
      returnToDos( 3 );
    }

    curIO=hystTotalIO++;

    hystIoInfo = (IOInfo **)realloc( hystIoInfo, hystTotalIO*sizeof(IOInfo **) );
    checkPtr( hystIoInfo );

    IOInfo *newIO( new IOInfo );
    checkPtr( newIO );

    hystIoInfo[curIO] = newIO;

    newIO->name = new char[ strlen(protocol) +1 ];
    checkPtr( newIO->name );
    strcpy( newIO->name, protocol );

    newIO->count =  0;

  }

  hystIoInfo[curIO]->count += count;

}

static near void registerNode( NodeInfo& node )
{

  for( index curNode=0; curNode<hystTotalNodes; curNode++ )
    if( hystNodes[curNode]->point==node.point &&
        hystNodes[curNode]->node==node.node &&
        hystNodes[curNode]->net==node.net &&
        hystNodes[curNode]->zone==node.zone &&
        !stricmp( hystNodes[curNode]->domain, node.domain ) )
      break;

  if( curNode==hystTotalNodes )
  {

    if( hystTotalNodes==MaxItems )
    {
      printf( "*Fatal* Too many nodes! Support up to %u nodes.\n",
              MaxItems );
      returnToDos( 1 );
    }

    curNode=hystTotalNodes++;

    void *work = malloc( hystTotalNodes*sizeof(NodeInfo **) );
    checkPtr( work );

    memcpy( work, hystNodes, (hystTotalNodes-1)*sizeof(NodeInfo **) );
    free( hystNodes );
    hystNodes = (NodeInfo **)work;

    NodeInfo *newNode( new NodeInfo );
    checkPtr( newNode );

    memset( newNode, 0, sizeof(NodeInfo) );
    hystNodes[curNode] = newNode;

    newNode->domain = new char[ strlen(node.domain) +1 ];
    checkPtr( newNode->domain );
    strcpy( newNode->domain, node.domain );

    newNode->zone = node.zone;
    newNode->net = node.net;
    newNode->node = node.node;
    newNode->point = node.point;

  }

  hystNodes[curNode]->ioTime     += node.ioTime    ;
  hystNodes[curNode]->sent       += node.sent      ;
  hystNodes[curNode]->received   += node.received  ;
  hystNodes[curNode]->income     += node.income    ;
  hystNodes[curNode]->outcome    += node.outcome   ;
  hystNodes[curNode]->inSession  += node.inSession ;
  hystNodes[curNode]->outSession += node.outSession;
  hystNodes[curNode]->bbs        += node.bbs       ;
  hystNodes[curNode]->inMail     += node.inMail    ;
  hystNodes[curNode]->inXmail    += node.inXmail   ;
  hystNodes[curNode]->inFiles    += node.inFiles   ;
  hystNodes[curNode]->outMail    += node.outMail   ;
  hystNodes[curNode]->outXmail   += node.outXmail  ;
  hystNodes[curNode]->outFiles   += node.outFiles  ;

}

static near void loadSecondHystory( char *hystoryName )
{
  hystFile = fopen( hystoryName, "rb" );

  if( hystFile==NULL )
  {
    IOerror( hystoryName );
    returnToDos(5);
  }

  printf( "Processing history file `%s'\n", hystoryName );

  HystHeader header;

  if( filelength( fileno( hystFile ) ) <
       sizeof(hystHeader) + sizeof(info) +
       sizeof(totalNodes) + sizeof(totalIO) )
    goto format;

  if( fread( &header, 1, sizeof(header), hystFile )!=sizeof(header) )
  {
cant_load:
    char *msg( "Can't process history" );

    if( feof(hystFile) )
      printf( "*Fatal* %s - unexpected EOF\n", msg );
    else
      IOerror( msg );

    returnToDos(5);
  }

  if( memcmp( header.sign, hystHeaderSign, sizeof(header.sign) ) )
  {
format:
    printf( "*Fatal* Illegal history file format\n" );
    returnToDos(5);
  }

  unsigned short crc = header.crc;
  register unsigned char *p( (unsigned char *)&header );

  header.crc = 0;
  for( size_t ipos=0; ipos<sizeof(header)-sizeof(header.crc); ipos++, p++ )
    header.crc += (unsigned short)*p;

  if( header.crc!=crc )
  {
    printf( "*Fatal* Illegal history file header CRC\n" );
    returnToDos(5);
  }

  if( header.init < hystHeader.init )
  {
    memcpy( hystHeader.hLogStart, header.hLogStart, sizeof(logStart) );
    hystHeader.init = header.init;
  }

  if( header.date > hystHeader.date )
  {
    memcpy( hystHeader.hLogEnd, header.hLogEnd, sizeof(logEnd) );
    hystHeader.date = header.date;
  }

  hystHeader.lines          += header.lines         ;
  hystHeader.size           += header.size          ;
  hystHeader.inSessions     += header.inSessions    ;
  hystHeader.outSessions    += header.outSessions   ;
  hystHeader.inHand         += header.inHand        ;
  hystHeader.outHand        += header.outHand       ;
  hystHeader.totalDialing   += header.totalDialing  ;
  hystHeader.totalAnswering += header.totalAnswering;
  hystHeader.totalIncome    += header.totalIncome   ;
  hystHeader.totalOutcome   += header.totalOutcome  ;
  hystHeader.totalExternal  += header.totalExternal ;
  hystHeader.totalBytesIn   += header.totalBytesIn  ;
  hystHeader.totalBytesOut  += header.totalBytesOut ;
  hystHeader.totalBBS       += header.totalBBS      ;
  hystHeader.totalTerminal  += header.totalTerminal ;
  hystHeader.inMail         += header.inMail        ;
  hystHeader.inXmail        += header.inXmail       ;
  hystHeader.inFiles        += header.inFiles       ;
  hystHeader.outMail        += header.outMail       ;
  hystHeader.outXmail       += header.outXmail      ;
  hystHeader.outFiles       += header.outFiles      ;
  hystHeader.sesFTS0001     += header.sesFTS0001    ;
  hystHeader.sesWaZOO       += header.sesWaZOO      ;
  hystHeader.sesEMSI        += header.sesEMSI       ;

  if( fread( &info, 1, sizeof(info), hystFile )!=sizeof(info) )
    goto cant_load;

  makeHeaderCRC();

  for( int i=0; i<48; i++ )
  {

    hystInfo[i].dial      += info[i].dial     ;
    hystInfo[i].answer    += info[i].answer   ;

    hystInfo[i].income    += info[i].income   ;
    hystInfo[i].outcome   += info[i].outcome  ;

    hystInfo[i].external  += info[i].external ;
    hystInfo[i].bbs       += info[i].bbs      ;
    hystInfo[i].terminal  += info[i].terminal ;

    hystInfo[i].usedCount += info[i].usedCount;

  }

  if( fread( &totalNodes, 1, sizeof(totalNodes), hystFile )!=sizeof(hystTotalNodes) )
    goto cant_load;

  for( index j=0; j<totalNodes; j++ )
  {

    NodeInfo newNode;

    if( fread( &newNode, 1, sizeof(NodeInfo), hystFile )!=sizeof(NodeInfo) )
      goto cant_load;

    char len( 0 );

    if( fread( &len, 1, sizeof(len), hystFile )!=sizeof(len) )
      goto cant_load;

    if( len )
    {
      newNode.domain = new char [len+1];
      checkPtr( newNode.domain );

      if( fread( newNode.domain, 1, len, hystFile )!=len )
        goto cant_load;

      newNode.domain[len] = 0;
    }
    else
      newNode.domain = newStr( "" );

    registerNode( newNode );
    delete newNode.domain;
  }

  if( fread( &totalIO, 1, sizeof(totalIO), hystFile )!=sizeof(totalIO) )
    goto cant_load;

  for( j=0; j<totalIO; j++ )
  {
    IOInfo newIO;

    if( fread( &newIO, 1, sizeof(IOInfo), hystFile )!=sizeof(IOInfo) )
      goto cant_load;

    char len( 0 );

    if( fread( &len, 1, sizeof(len), hystFile )!=sizeof(len) )
      goto cant_load;

    if( len )
    {
      newIO.name = new char [len+1];
      checkPtr( newIO.name );

      if( fread( newIO.name, 1, len, hystFile )!=len )
        goto cant_load;

      newIO.name[len] = 0;
    }
    else
      newIO.name = newStr( "*Unloaded*" );

    registerIOCount( newIO.name, newIO.count );
    delete newIO.name;

  }

  int k;
  for( i=0, k=0; i<strlen(finitaLaComedia); i++ )
    if( (k=fgetc( hystFile ),k)==EOF || char(k)!=finitaLaComedia[i] )
      goto format;

  if( fgetc( hystFile )!=EOF )
  {
    printf( "*Fatal* Too long history file\n" );
    returnToDos(5);
  }

  fclose( hystFile );
}

int doMerge( int argn, char **argc )
{

  if( argn<4 || stricmp( argc[1], "History" ) )
    return 1;

  char *mm( new char [ strlen(argc[0])+strlen(argc[1])+2 ] );

  checkPtr( mm );

  capitalize( argc[0] );
  capitalize( argc[1] );
  strcpy( mm, argc[0] );
  strcat( mm, " " );
  strcat( mm, argc[1] );
  argc[1] = mm;
  argc++;
  argn--;

  int option;

  while( (option=getopt( argn, argc, "h:" ))!=-1 )
    if( option=='h' )
    {
      hystName = optarg;
      capitalize( hystName );
    }
    else
    {
      putchar( '\n' );
      return 2;
    }

  if( optind+1>=argn )
  {
    printf( "*Fatal* Not enough files specified!\n\n" );
    return 2;
  }

  if( hystName==NULL )
    hystName = argc[optind];

  capitalize( hystName );

  for( int howFilesProcessed(0); optind<argn; optind++ )
  {
    char *current( argc[optind] );

    capitalize( current );

    if( access( current, 4 ) )
      printf( "File `%s' inaccessible - skipped!\n", current );
    else
      if( ++howFilesProcessed==1 )
      {
        loadHystory( current );
        fclose( hystFile );
      }
      else
        loadSecondHystory( current );
  }

  if( howFilesProcessed )
  {
    printf( "Total %d history files processed\n", howFilesProcessed );
    printf( "Initialize history file `%s'\n", hystName );

    totalNodes = hystTotalNodes;
    nodesInfo = hystNodes;

    totalIO = hystTotalIO;
    ioInfo = hystIoInfo;

    memcpy( info, hystInfo, sizeof(info) );

    hystFile = fopen( hystName, "w+b" );

    if( hystFile==NULL )
    {
      IOerror( hystName );
      returnToDos(5);
    }

    setbuf( hystFile, NULL );
    storeHystory();
  }
  else
    printf( "No files processed - output history file not %s\n",
            access( hystName, 0 ) ? "created" : "changed" );

  runStat();

  return 0;
}

