#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <dos.h>
#include <signal.h>
#include <float.h>

#include "busy.h"

#ifdef __OS2__
#define cdecl __stdcall
#endif

static cdecl int int23()
{
  printf( "\n*Int23* Breaking, Ctrl-Break or Ctrl-C pressed\n" );

  raise( SIGABRT );
  return 0;
}

#if !defined(__OS2__)
static cdecl int int24( int, int, int, int )
{
  printf( "\n*Int24* Critical error encountred\n" );

  raise( SIGABRT );
  return( _HARDERR_ABORT );
}
#endif

static cdecl void signalTERM( int )
{
  printf( "\n*SIGTERM* Program terminated by another process\n" );

  returnToDos( 13 );
}

static cdecl void signalABRT( int )
{
  printf( "\n*SIGABRT* Program was aborted by previous error\n" );

  returnToDos( 100 );
}

static void printPos( int code, int realSig, int raiseSig, int *regs )
{
  printf( "*Debug* " );

  if( code==raiseSig )
    printf( "Signal was raised!\n" );
  else
    if( code==realSig )
    {
#if !defined(__OS2__)

      printf( "Error location is %04X:%04X\n", regs[10], regs[9] );
      printf( "\tAX=%04X BX=%04X CX=%04X DX=%04X\n",
	      regs[8], regs[7], regs[6], regs[5] );
      printf( "\tDS=%04X ES=%04X DI=%04X SI=%04X\n",
	      regs[3], regs[4], regs[1], regs[2] );
      printf( "\tBP=%04X Flags=%04X\n", regs[0], regs[11] );

#else

      printf( "Error location is %04X:%08X\n", regs[10], regs[9] );
      printf( "\tEAX=%08X   EBX=%08X ECX=%08X EDX=%08X\n",
	      regs[8], regs[7], regs[6], regs[5] );
      printf( "\t DS=%04X        ES=%04X     EDI=%08X ESI=%08X\n",
	      regs[3], regs[4], regs[1], regs[2] );
      printf( "\tEBP=%08X Flags=%04X\n", regs[0], regs[11] );

#endif
    }
    else
      printf( "Sorry, no more information available! :-(\n" );

}

static cdecl void signalILL( int, int subCode, int *regs )
{
#if !defined(__OS2__)

  printf( "\n*SIGILL* Illegal instruction executed\n" );
  printPos( subCode, ILL_EXECUTION, ILL_EXPLICITGEN, regs );

#else

  char *type( NULL );

  switch( subCode )
  {
  case ILL_EXPLICITGEN:
    printPos( subCode, 0, ILL_EXPLICITGEN, 0 );
    break;

  case ILL_EXECUTION:
    type = "Illegal instruction executed";
    break;

  case ILL_PRIVILEGED:
    type = "Privileged instruction executed";
    break;

  default:
    type = "Unknown SIGILL exception";
  }

  if( type!=NULL )
  {
    printf( "\n*SIGILL* %s\n", type );
    printPos( subCode, subCode, 0, regs );
  }

#endif

  raise( SIGABRT );
  returnToDos( 100 );
}

static cdecl void signalSEGV( int, int subCode, int *regs )
{
#if !defined(__OS2__)

  printf( "\n*SIGSEGV* BOUND instruction constraint exception\n" );
  printPos( subCode, SEGV_BOUND, SEGV_EXPLICITGEN, regs );

#else

  char *type( NULL );

  switch( subCode )
  {
  case SEGV_EXPLICITGEN:
    printPos( subCode, 0, SEGV_EXPLICITGEN, 0 );
    break;

  case SEGV_BOUND:
    type = "BOUND instruction constraint exception";
    break;

  case SEGV_ACCESS:
    type = "Memory access violation";
    break;

  case SEGV_STACK:
    type = "Unable to grow stack";
    break;

  default:
    type = "Unknown SIGSEGV exception";
  }

  if( type!=NULL )
  {
    printf( "\n*SIGSEGV* %s\n", type );
    printPos( subCode, subCode, 0, regs );
  }

#endif

  raise( SIGABRT );
  returnToDos( 100 );
}

static cdecl void signalFPE( int, int subCode, int *regs )
{
  printf( "\n*SIGFPE* Illegal ariphmetic operation\n" );

  {
    char *type( NULL );

    switch( subCode )
    {
    case FPE_EXPLICITGEN:
      printPos( subCode, 0, FPE_EXPLICITGEN, 0 );
      break;

    case FPE_INTDIV0:
    case FPE_INTOVFLOW:
      printf( "*SIGFPE* %s\n", subCode==FPE_INTDIV0 ?
					 "Integer division by zero" :
					 "Integer overflow" );

      printPos( subCode, subCode, 0, regs );
      break;


    case FPE_INEXACT:
      type = "Floating point precision fault";
      break;

    case FPE_INVALID:
      type = "Invalid floating point operation";
      break;

    case FPE_OVERFLOW:
      type = "Floating point overflow";
      break;

    case FPE_UNDERFLOW:
      type = "Floating point underflow";
      break;

    case FPE_ZERODIVIDE:
      type = "Floating point division by zero";
      break;

#if defined(__OS2__)
    case FPE_STACKFAULT:
      type = "Floating point stack overflow/underflow";
      break;
#endif

    default:
      type = "Unknown SIGFPE exception";
    }

    if( type!=NULL )
    {
      printf( "*SIGFPE* %s\n", type );
      printPos( 1, 0, 0, 0 );
    }

  }

  raise( SIGABRT );
  returnToDos( 100 );
}

void setSignals()
{
#if defined(__cplusplus)
  typedef cdecl void (*fptrSig)(int);
#else
  typedef cdecl void (*fptrSig)();
#endif

#if !defined(__OS2__)
  harderr( int24 );
  ctrlbrk(int23 );
#else
  signal( SIGINT,   (fptrSig)int23      );
  signal( SIGBREAK, (fptrSig)int23      );
#endif

  signal( SIGFPE,   (fptrSig)signalFPE  );
  signal( SIGILL,   (fptrSig)signalILL  );
  signal( SIGSEGV,  (fptrSig)signalSEGV );
  signal( SIGABRT,  (fptrSig)signalABRT );
  signal( SIGTERM,  (fptrSig)signalTERM );

}

#pragma startup setSignals 64
